/*
 *  RTL8139IO.cpp
 *  PCGenRTL8139Ethernet
 *
 *  Created by orByte on 07/01/2007.
 *  Copyright 2007. All rights reserved.
 *
 * APSL (Apple Public Source License)
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License."
 *
 */
 
/*
 * Copyright (c) 2007 orByte.  All rights reserved. 
 *
 * rtl8139io.cpp
 *
 * HISTORY
 *
 * 09-Jan-07	orByte, added following features:
 *				(+) Run-time allocation of MMIO/PIO access mode
 *				(+) Option to override PIO, and use MMIO instead
 *		
 *
 */

#include "RTL8139.h"


//---------------------------------------------------------------------------
//
//						** class ioAccessor **
//
//---------------------------------------------------------------------------
#undef super
#define super OSObject
OSDefineMetaClassAndAbstractStructors ( ioAccessor, OSObject )
//---------------------------------------------------------------------------
// Function: init
//
// Purpose:
//   object initialization.

bool ioAccessor::init ( )
{
	if (!super::init()) {
        return false;
    }
	
    return true;
}


//---------------------------------------------------------------------------
//
//						** class ioAccessorPIO **
//
//---------------------------------------------------------------------------
#undef super
#define super ioAccessor
OSDefineMetaClassAndStructors ( ioAccessorPIO, ioAccessor )
//---------------------------------------------------------------------------
// Functions: init, withIOMode
//
// Purpose:
//   object initialization.

bool ioAccessorPIO::init ( IOPCIDevice * pNub, IOMemoryMap * ioMap )
{
	if (!super::init()) {
        return false;
    }
	
    pciNub = pNub;
	csrMap = ioMap;
    return true;
}


ioAccessor * ioAccessorPIO::withIOMode ( IOPCIDevice * pNub, IOMemoryMap * ioMap )
{
	ioAccessorPIO * ioMode = new ioAccessorPIO;
	
	if ( ioMode && !ioMode->init ( pNub, ioMap ) ) {
        ioMode->release ();
        return 0;
    }
    return ( ioAccessor * ) ioMode;
}

//---------------------------------------------------------------------------
// Functions: csrWrite32, csrWrite32Slow
//
// Purpose:
//   Write 32 bits to csr using PIO.

void ioAccessorPIO::csrWrite32 ( UInt16 offset, UInt32 value )
{
	pciNub->ioWrite32 ( offset, value, csrMap );
}

void ioAccessorPIO::csrWrite32Slow ( UInt16 offset, UInt32 value )
{
	pciNub->ioWrite32 ( offset, value, csrMap );
	pciNub->ioRead32 ( offset, csrMap );
}


//---------------------------------------------------------------------------
// Functions: csrWrite16, csrWrite16Slow
//
// Purpose:
//   Write 16 bits to csr using PIO.

void ioAccessorPIO::csrWrite16 ( UInt16 offset, UInt16 value )
{
	pciNub->ioWrite16 ( offset, value, csrMap );
}

void ioAccessorPIO::csrWrite16Slow ( UInt16 offset, UInt16 value )
{
	pciNub->ioWrite16 ( offset, value, csrMap );
	pciNub->ioRead16 ( offset, csrMap );
}

//---------------------------------------------------------------------------
// Functions: csrWrite8, csrWrite8Slow
//
// Purpose:
//   Write 8 bits to csr using PIO.

void ioAccessorPIO::csrWrite8 ( UInt16 offset, UInt8 value )
{
	pciNub->ioWrite8 ( offset, value, csrMap );
}

void ioAccessorPIO::csrWrite8Slow ( UInt16 offset, UInt8 value )
{
	pciNub->ioWrite8 ( offset, value, csrMap );
	pciNub->ioRead8 ( offset, csrMap );
}

//---------------------------------------------------------------------------
// Function: csrRead32
//
// Purpose:
//   Read 32 bits from csr using PIO.

UInt32 ioAccessorPIO::csrRead32 ( UInt16 offset )
{
	return pciNub->ioRead32 ( offset, csrMap );
}

//---------------------------------------------------------------------------
// Function: csrRead16
//
// Purpose:
//   Read 16 bits from csr using PIO.

UInt16 ioAccessorPIO::csrRead16 ( UInt16 offset )
{
	return pciNub->ioRead16 ( offset, csrMap );
}

//---------------------------------------------------------------------------
// Function: csrRead8
//
// Purpose:
//   Read 8 bits from csr using PIO.

UInt8 ioAccessorPIO::csrRead8 ( UInt16 offset )
{
	return pciNub->ioRead8 ( offset, csrMap );
}



//---------------------------------------------------------------------------
//
//						** class ioAccessorMMIO **
//
//---------------------------------------------------------------------------
OSDefineMetaClassAndStructors ( ioAccessorMMIO, ioAccessor )
//---------------------------------------------------------------------------
// Functions: init, withIOMode
//
// Purpose:
//   object initialization.

bool ioAccessorMMIO::init ( volatile void * cBase )
{
	if (!super::init()) {
        return false;
    }
	
	csrBase = cBase;
    return true;
}


ioAccessor * ioAccessorMMIO::withIOMode ( volatile void * cBase )
{
	ioAccessorMMIO * ioMode = new ioAccessorMMIO;
	
	if ( ioMode && !ioMode->init ( cBase ) ) {
        ioMode->release ();
        return 0;
    }
    return ioMode;
}

//---------------------------------------------------------------------------
// Functions: csrWrite32, csrWrite32Slow
//
// Purpose:
//   Write 32 bits to csr using MMIO.

void ioAccessorMMIO::csrWrite32 ( UInt16 offset, UInt32 value )
{
	 OSWriteLittleInt32 ( (void *) csrBase, offset, value );
	 OSSynchronizeIO ();
}

void ioAccessorMMIO::csrWrite32Slow ( UInt16 offset, UInt32 value )
{
	 OSWriteLittleInt32 ( (void *) csrBase, offset, value );
	 OSSynchronizeIO ();
	 OSReadLittleInt32 ( ( void * ) csrBase, offset );
}


//---------------------------------------------------------------------------
// Functions: csrWrite16, csrWrite16Slow
//
// Purpose:
//   Write 16 bits to csr using MMIO.

void ioAccessorMMIO::csrWrite16 ( UInt16 offset, UInt16 value )
{
	 OSWriteLittleInt16 ( (void *) csrBase, offset, value );
	 OSSynchronizeIO ();
}

void ioAccessorMMIO::csrWrite16Slow ( UInt16 offset, UInt16 value )
{
	 OSWriteLittleInt16 ( (void *) csrBase, offset, value );
	 OSSynchronizeIO ();
	 OSReadLittleInt16 ( ( void * ) csrBase, offset );
}

//---------------------------------------------------------------------------
// Functions: csrWrite8, csrWrite8Slow
//
// Purpose:
//   Write 8 bits to csr using MMIO.

void ioAccessorMMIO::csrWrite8 (  UInt16 offset, UInt8 value )
{
	* ( volatile uint8_t * )( ( uintptr_t ) csrBase + offset ) = value;
	OSSynchronizeIO();
}

void ioAccessorMMIO::csrWrite8Slow (  UInt16 offset, UInt8 value )
{
	* ( volatile uint8_t * )( ( uintptr_t ) csrBase + offset ) = value;
	OSSynchronizeIO();
	* ( volatile uint8_t * )( ( uintptr_t ) csrBase + offset);
}

//---------------------------------------------------------------------------
// Function: csrRead32
//
// Purpose:
//   Read 32 bits from csr using MMIO.

UInt32 ioAccessorMMIO::csrRead32( UInt16 offset )
{
	return OSReadLittleInt32 ( ( void * ) csrBase, offset );
}

//---------------------------------------------------------------------------
// Function: csrRead16
//
// Purpose:
//   Read 16 bits from csr using MMIO.

UInt16 ioAccessorMMIO::csrRead16( UInt16 offset )
{
	return OSReadLittleInt16 ( ( void * ) csrBase, offset );
}

//---------------------------------------------------------------------------
// Function: csrRead8
//
// Purpose:
//   Read 8 bits from csr using MMIO.

UInt8 ioAccessorMMIO::csrRead8( UInt16 offset )
{
	return * ( volatile uint8_t * )( ( uintptr_t ) csrBase + offset);
}



