/*
 *  RTL8139IO.h
 *  PCGenRTL8139Ethernet
 *
 *  Created by orByte on 07/01/2007.
 *  Copyright 2007. All rights reserved.
 *
 * APSL (Apple Public Source License)
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License."
 *
 */
 
/*
 * Copyright (c) 2007 orByte.  All rights reserved. 
 *
 * rtl8139io.h
 *
 * HISTORY
 *
 * 09-Jan-07	orByte, added following features:
 *				(+) Run-time allocation of MMIO/PIO access mode
 *				(+) Option to override PIO, and use MMIO instead
 *		
 *
 */
 
 #ifndef _RTL8139IO_H
 #define _RTL8139IO_H


 class ioAccessor : public OSObject
 {
 
	OSDeclareAbstractStructors ( ioAccessor )
	
	public:
	
	virtual void				csrWrite32 ( UInt16 offset, UInt32 value ) = 0;
    virtual void				csrWrite16 ( UInt16 offset, UInt16 value ) = 0;
    virtual void				csrWrite8 (  UInt16 offset, UInt8 value ) = 0;
	virtual void				csrWrite32Slow ( UInt16 offset, UInt32 value ) = 0;
    virtual void				csrWrite16Slow ( UInt16 offset, UInt16 value ) = 0;
    virtual void				csrWrite8Slow (  UInt16 offset, UInt8 value ) = 0;
    virtual UInt32				csrRead32 ( UInt16 offset ) = 0;
    virtual UInt16				csrRead16 ( UInt16 offset ) = 0;
    virtual UInt8				csrRead8 (  UInt16 offset ) = 0;
	
	protected:
	virtual bool				init ( );

 };
 
  class ioAccessorPIO : public ioAccessor
 {
 
	OSDeclareDefaultStructors ( ioAccessorPIO )
	
	private:
	
	IOPCIDevice *				pciNub;
	IOMemoryMap	*				csrMap;

	public:

	virtual void				csrWrite32 ( UInt16 offset, UInt32 value );
    virtual void				csrWrite16 ( UInt16 offset, UInt16 value );
    virtual void				csrWrite8 (  UInt16 offset, UInt8 value );
	virtual void				csrWrite32Slow ( UInt16 offset, UInt32 value );
    virtual void				csrWrite16Slow ( UInt16 offset, UInt16 value );
    virtual void				csrWrite8Slow (  UInt16 offset, UInt8 value );
    virtual UInt32				csrRead32 ( UInt16 offset );
    virtual UInt16				csrRead16 ( UInt16 offset );
    virtual UInt8				csrRead8 (  UInt16 offset );
	virtual ioAccessor *		withIOMode ( IOPCIDevice * pNub, IOMemoryMap * ioMap );
	
	protected:
	
	virtual bool				init ( IOPCIDevice * pNub, IOMemoryMap * ioMap );
	
	

 };
 
  class ioAccessorMMIO : public ioAccessor
 {
 
	OSDeclareDefaultStructors ( ioAccessorMMIO )
	
	private:
	
	volatile void *				csrBase;
 
	public:

	virtual void				csrWrite32 ( UInt16 offset, UInt32 value );
    virtual void				csrWrite16 ( UInt16 offset, UInt16 value );
    virtual void				csrWrite8 (  UInt16 offset, UInt8 value );
	virtual void				csrWrite32Slow ( UInt16 offset, UInt32 value );
    virtual void				csrWrite16Slow ( UInt16 offset, UInt16 value );
    virtual void				csrWrite8Slow (  UInt16 offset, UInt8 value );
    virtual UInt32				csrRead32 ( UInt16 offset );
    virtual UInt16				csrRead16 ( UInt16 offset );
    virtual UInt8				csrRead8 (  UInt16 offset );
	virtual ioAccessor *		withIOMode ( volatile void * cBase );
	
	protected:
	
	virtual bool				init ( volatile void * cBase );
	

 };


 #endif /* !_RTL8139IO_H */
